@@ -149,4 +149,17 @@ public interface SAMLConfig {
      */
     boolean isAutoLoginEnabled();
 
+    /**
+     * Gets Whether SAML unsolicited responses are enabled.
+     *
+     * @return <code>true</code> if unsolicited responses are on.
+     */
+    boolean isAllowUnsolicitedResponses();
+
+    /**
+     * Gets whether SAML-specific auto-login is enabled, that uses the SessionIndex of the AuthnResponse.
+     *
+     * @return <code>true</code> if auto-login is on for SessionIndex.
+     */
+    boolean isSessionIndexAutoLoginEnabled();
 }
