@@ -50,8 +50,10 @@
 package com.openexchange.saml.impl;
 
 import static com.openexchange.saml.SAMLProperties.ACS_URL;
+import static com.openexchange.saml.SAMLProperties.ALLOW_UNSOLICITED_RESPONSES;
 import static com.openexchange.saml.SAMLProperties.ENABLE_AUTO_LOGIN;
 import static com.openexchange.saml.SAMLProperties.ENABLE_METADATA_SERVICE;
+import static com.openexchange.saml.SAMLProperties.ENABLE_SESSION_INDEX_AUTO_LOGIN;
 import static com.openexchange.saml.SAMLProperties.ENABLE_SINGLE_LOGOUT;
 import static com.openexchange.saml.SAMLProperties.ENTITY_ID;
 import static com.openexchange.saml.SAMLProperties.IDP_ENTITY_ID;
@@ -99,6 +101,9 @@ public class DefaultConfig implements SAMLConfig {
 
     private boolean autoLoginEnabled;
 
+    private boolean allowUnsolicitedResponses;
+
+    private boolean sessionIndexAutoLoginEnabled;
 
     private DefaultConfig() {
         super();
@@ -125,6 +130,8 @@ public class DefaultConfig implements SAMLConfig {
 
         config.setEnableMetadataService(configService.getBoolProperty(ENABLE_METADATA_SERVICE, false));
         config.setAutoLoginEnabled(configService.getBoolProperty(ENABLE_AUTO_LOGIN, false));
+        config.setAllowUnsolicitedResponses(configService.getBoolProperty(ALLOW_UNSOLICITED_RESPONSES, true));
+        config.setSessionIndexAutoLoginEnabled(configService.getBoolProperty(ENABLE_SESSION_INDEX_AUTO_LOGIN, false));
         return config;
     }
 
@@ -256,4 +263,21 @@ public class DefaultConfig implements SAMLConfig {
         this.autoLoginEnabled = autoLoginEnabled;;
     }
 
+    @Override
+    public boolean isAllowUnsolicitedResponses() {
+        return allowUnsolicitedResponses;
+    }
+
+    public void setAllowUnsolicitedResponses(boolean allowUnsolicitedResponses) {
+        this.allowUnsolicitedResponses = allowUnsolicitedResponses;
+    }
+
+    @Override
+    public boolean isSessionIndexAutoLoginEnabled() {
+        return sessionIndexAutoLoginEnabled;
+    }
+
+    private void setSessionIndexAutoLoginEnabled(boolean sessionIndexAutoLoginEnabled) {
+        this.sessionIndexAutoLoginEnabled = sessionIndexAutoLoginEnabled;
+    }
 }
