@@ -235,17 +235,10 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
             throw SAMLExceptionCode.UNSUPPORTED_BINDING.create(binding.name());
         }
 
-        String relayState = httpRequest.getParameter("RelayState");
-        if (relayState == null) {
-            throw SAMLExceptionCode.INVALID_REQUEST.create("The 'RelayState' parameter was not set");
-        }
-
-        AuthnRequestInfo requestInfo = stateManagement.removeAuthnRequestInfo(relayState);
-        if (requestInfo == null) {
-            throw SAMLExceptionCode.INVALID_REQUEST.create("The 'RelayState' parameter was invalid");
-        }
-
         Response response = extractAuthnResponse(httpRequest);
+
+        AuthnRequestInfo requestInfo = getRequestInfo(httpRequest, response);
+
         try {
             ValidationStrategy validationStrategy = backend.getValidationStrategy(config, stateManagement);
             AuthnResponseValidationResult validationResult = validationStrategy.validateAuthnResponse(response, requestInfo, binding);
@@ -295,6 +288,63 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
         }
     }
 
+    /**
+     * Helper method, that tries to fetch the RequestInfo from the WebSSOCustomizer and falls back to default
+     * @param httpRequest the HttpServletRequest
+     * @param response The authnResponse
+     * @return the AuthnRequestInfo
+     * @throws OXException
+     */
+    private AuthnRequestInfo getRequestInfo(HttpServletRequest httpRequest, Response response) throws OXException {
+        AuthnRequestInfo customGetRequestInfo = customGetRequestInfo(httpRequest, response);
+        if (null != customGetRequestInfo) {
+            return customGetRequestInfo;
+        }
+        AuthnRequestInfo requestInfo;
+        if (response.getInResponseTo() == null) {
+            /*
+             * An unsolicited <Response> MUST NOT contain an InResponseTo attribute, nor should any bearer
+             * <SubjectConfirmationData> elements contain one. If metadata as specified in [SAMLMeta] is used,
+             * the <Response> or artifact SHOULD be delivered to the <md:AssertionConsumerService> endpoint
+             * of the service provider designated as the default.
+             * [profiles 06 - 4.1.5 p20]
+             */
+            if (config.isAllowUnsolicitedResponses()) {
+                String relayState = httpRequest.getParameter("RelayState");
+                if (relayState == null) {
+                    // use DefaultAuthnRequestInfo if no RelayState is set
+                    String domainName = getDomainName(httpRequest);
+                    requestInfo = new DefaultAuthnRequestInfo();
+                    ((DefaultAuthnRequestInfo)requestInfo).setDomainName(domainName);
+                } else {
+                    requestInfo = backend.parseRelayState(response, relayState);
+                }
+            } else {
+                throw SAMLExceptionCode.INVALID_REQUEST.create("Unsolicited responses are not enabled");
+            }
+        } else {
+            String relayState = httpRequest.getParameter("RelayState");
+            if (relayState == null) {
+                throw SAMLExceptionCode.INVALID_REQUEST.create("The 'RelayState' parameter was not set");
+            }
+
+            requestInfo = stateManagement.removeAuthnRequestInfo(relayState);
+            if (requestInfo == null) {
+                throw SAMLExceptionCode.INVALID_REQUEST.create("The 'RelayState' parameter was invalid");
+            }
+        }
+        return requestInfo;
+    }
+
+    private AuthnRequestInfo customGetRequestInfo(HttpServletRequest httpRequest, Response response) throws OXException {
+        WebSSOCustomizer customizer = backend.getWebSSOCustomizer();
+        if (customizer != null) {
+            return customizer.getRequestInfo(httpRequest, response, stateManagement);
+        }
+
+        return null;
+    }
+
     @Override
     public String buildLogoutRequest(HttpServletRequest httpRequest, HttpServletResponse httpResponse, Session session) throws OXException {
         final LogoutRequest logoutRequest = customizeLogoutRequest(prepareLogoutRequest(session), session, httpRequest, httpResponse);
@@ -334,7 +384,7 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
         LogoutResponse response = extractLogoutResponse(httpRequest, binding);
         try {
             ValidationStrategy validationStrategy = backend.getValidationStrategy(config, stateManagement);
-            validationStrategy.validateLogoutResponse(response, requestInfo, binding);
+            validationStrategy.validateLogoutResponse(response, httpRequest, requestInfo, binding);
             URI redirectLocationBuilder = new URIBuilder()
                 .setScheme(getRedirectScheme(httpRequest))
                 .setHost(requestInfo.getDomainName())
@@ -862,7 +912,7 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
         try {
             if (binding == Binding.HTTP_REDIRECT) {
                 // bytes are deflated in redirect binding
-                return openSAML.getParserPool().parse(new InflaterInputStream(new ByteArrayInputStream(requestBytes)));
+                return openSAML.getParserPool().parse(new InflaterInputStream(new ByteArrayInputStream(requestBytes), new Inflater(true)));
             } else {
                 return openSAML.getParserPool().parse(new ByteArrayInputStream(requestBytes));
             }
